@@ -1,12 +1,40 @@
-## drop.tip.R (2017-07-27)
+## drop.tip.R (2018-06-21)
 
 ##   Remove Tips in a Phylogenetic Tree
 
-## Copyright 2003-2017 Emmanuel Paradis, 2017 Klaus Schliep
+## Copyright 2003-2017 Emmanuel Paradis, 2017-2018 Klaus Schliep, 2018 Joseph Brown
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
 
+keep.tip <- function(phy, tip)
+{
+    if (!inherits(phy, "phylo"))
+        stop("object \"phy\" is not of class \"phylo\"")
+    Ntip <- length(phy$tip.label)
+    ## convert to indices if strings passed in
+    if (is.character(tip)) {
+        idx <- match(tip, phy$tip.label)
+        ## stop on bad tip names
+        ## alternative is to warn but proceed. not sure what stance is
+        if (anyNA(idx)) {
+            um <- c("umatched tip labels:\n", paste(tip[is.na(idx)], collapse = " "))
+            stop(um)
+        }
+        tip <- idx
+    } else {
+        # check that passed in indices are all valid
+        out.of.range <- tip > Ntip
+        if (any(out.of.range)) {
+            warning("some tip numbers were larger than the number of tips: they were ignored")
+            tip <- tip[!out.of.range]
+        }
+    }
+    ## get complement tip indices to drop
+    toDrop <- setdiff(1:Ntip, tip)
+    drop.tip(phy, toDrop)
+}
+
 extract.clade <- function(phy, node, root.edge = 0, collapse.singles = TRUE, interactive = FALSE)
 {
     n <- length(phy$tip.label)
@@ -67,8 +95,10 @@ drop.tip <-
     if (!length(tip)) return(phy)
 
     if (length(tip) == Ntip) {
-        warning("drop all tips of the tree: returning NULL")
-        return(NULL)
+        if (Nnode(phy) < 3 || trim.internal) { # by Klaus (2018-06-21)
+            warning("drop all tips of the tree: returning NULL")
+            return(NULL)
+        }
     }
 
     wbl <- !is.null(phy$edge.length)
@@ -171,18 +201,23 @@ drop.tip <-
     ## the tips may not be sorted in increasing order in the
     ## 2nd col of edge, so no need to reorder $tip.label
     phy$edge[TERMS, 2] <- rank(phy$edge[TERMS, 2])
-    phy$tip.label <- phy$tip.label[-tip]
+    ## fix by Thomas Sibley (2017-10-28):
+    if (length(tip)) phy$tip.label <- phy$tip.label[-tip]
 
     ## make new tip labels if necessary:
     if (subtree || !trim.internal) {
         ## get the numbers of the nodes that become tips:
         node2tip <- oldNo.ofNewTips[oldNo.ofNewTips > Ntip]
-        new.tip.label <- if (subtree) {
-            paste("[", N[node2tip], "_tips]", sep = "")
-        } else {
-            if (is.null(phy$node.label)) rep("NA", length(node2tip))
-            else phy$node.label[node2tip - Ntip]
-        }
+        ## fix by Thomas Sibley (2017-10-28):
+        new.tip.label <-
+            if (!length(node2tip)) {
+                character(0)
+            } else if (subtree) {
+                paste("[", N[node2tip], "_tips]", sep = "")
+            } else {
+                if (is.null(phy$node.label)) rep("NA", length(node2tip))
+                else phy$node.label[node2tip - Ntip]
+            }
 #        if (!is.null(phy$node.label))
 #            phy$node.label <- phy$node.label[-(node2tip - Ntip)]
         phy$tip.label <- c(phy$tip.label, new.tip.label)
